(**-- STDNODE.pas ---------------------------------------------------1993----*
* Standard TreeNode-object definition                                        *
* developed by Rik Smit on 20 februari 1993.                                 *
* last updates: 21 mei 1994 , 3 jan 1998                                     *
* Used by Lazarus 2.0.6 AND Delphi Rio 10.3.3 FTree.exe                      *
**-------------------------------------------------------------------------**)

(*4--Code 4: The 1993 Tree Index Object definition Blaise_86_UK-------------*)
Unit STDNODE;

interface

type PTrndx = ^TTrndx;
     TTrndx = OBJECT                                { tree-index object type }
       dpth,                                                    { tree depth }
       pscd,                         { tree position code, bit wise oriented }
       nmcd  : Byte;                                      { node number code }

       constructor Init(Adpth, Apscd, Anmcd : Byte);
       destructor  Done; Virtual;

       function    Givedpth : Byte;
       function    Givepscd : Byte;
       function    Givenmcd : Byte;

       procedure   Setnmcd(Anmcd: Byte);                   { set number code }
       function    NdInfoStr(mxdpth: word):String;        { node info string }

     END;
(*-------------------------------------------------------------------------4*)

     binstr  = string[8];  // binary byte number display string (max 8 digits)

     function BitSet      (pos, item: byte)       :Boolean;
     function SetBit      (pos, item: byte)       :Byte;
     function ClearBit    (pos, item: byte)       :Byte;

     function GiveRank    (Apscd,Adpth: Byte)     :Byte;
     function GiveRankPscd(Arnk, Adpth: Byte)     :Byte;
     function GiveBinstr  (b, d: byte)            :binstr;

implementation
var
LST : Textfile;
(**------------------------------------------------------------------------**)

  (** BitSet checks whether the specified position        *
    * in the item has the value 1.                   1993 **)

FUNCTION BitSet(pos: byte; item: byte): Boolean;
  var bt : Byte;
BEGIN
  bt    := $01;                                            { bt get 00000001 }
  if Pos > 1 then                                               { limitation }
    bt    := bt SHL (Pos-1);               { move bit no 1 pos-1 to the left }
  BitSet := (item and bt) > 0;    { check whether both bits have the value 1 }
END;
(**------------------------------------------------------------------------**)

  (** SetBit returns the bit 1 at the specified location (pos)       *
    * in the specified variable (Item).                       1993   **)

FUNCTION SetBit(pos, item: byte): Byte;
  var bt : Byte;
 BEGIN
  bt := $01;            { bt ==> 0000 0001 }
  bt := bt SHL (Pos-1); { bt ==> 0001 0000  as pos = 5: ==> put 4 zeros in front of it }
  SetBit := item or bt; { Result ==> ???1 ???? depending on the content of item }
END;
(**------------------------------------------------------------------------**)

  (** ClearBit returns the bit 0 at the specified location    *
    * (pos) in the specified variable (Item).          1993   **)

FUNCTION ClearBit(pos, item: byte): Byte;
  var bt : Byte;
BEGIN
  bt := $01;
  if Pos > 1 then                 { protection agains zero or negative number}
    bt := bt SHL (Pos-1); { move the first bit (pos-1) positions to the left }
  if (item and bt) > 0 then
(* bit is gezet en kan dus op nul worden gezet *)
    ClearBit := (item xor bt)   // the bit is here 1, so returns the bit 0
  else ClearBit := item;
END;
(**------------------------------------------------------------------------**)

(** GiveRank convert the pscd byte value into a tree depth        *
*    dependent ranking value that is numbered ascending           *
*    from top to bottom, starting with the value 1. date: 1993   **)

FUNCTION GiveRank(Apscd, Adpth: Byte): Byte;
  var i,j : Byte; { tellers }
BEGIN
  j := 0;
// If from Hi to Lo the bit in the pscd has the value one ...
//    then sum the read byte value with function result.
  for i:= Adpth downto 1 do
    if BitSet(i,Apscd) then Inc(j,SetBit((Adpth-i+1),0));
  GiveRank := j+1;  { compensate the function result for the start value zero}
  // Result := j+i;
END;                                                              { GiveRank }
(**------------------------------------------------------------------------**)

(** GiveRankPscd: The reverse function of GiveRank  dd 13 okt 2003 **)

FUNCTION GiveRankPscd(Arnk, Adpth: Byte): Byte;
  var i,Apscd : Byte; { tellers }
BEGIN
  Apscd := 0;
  DEC(Arnk);        { compensate Arnk for the start value zero, see GiveRank }
  for i := 1 to Adpth do
  BEGIN
    if BitSet(i,Arnk) then
      Apscd := SetBit(( Adpth-i+1),Apscd);
  END;
  GiveRankPscd := APscd;              // Result := APscd
END;                                                          { GiveRankPscd }
(**------------------------------------------------------------------------**)

  (** GiveBinstr converts a byte into a string of ones en zeros. *
    * b is the byte to be converted,                             *
    * d indicates the number of bits put in a string. Max 8.     **)

FUNCTION GiveBinstr(b,d: byte): binstr;
  var i, bt : byte;
      s : binstr;
BEGIN
  s := '';
  bt := $01;
  for i := 1 to d do
  BEGIN
    if (b and bt) > 0 then s := '1'+s   // there is a 1 in b
                      else s := '0'+s;  // there is a 0 in b
    bt := bt shl 1;                     // move bits one position to the left
  END;
  GiveBinstr := s;                      // Result := s;
END;

(**  TTrNdx  --------------------------------------------------------------**)

CONSTRUCTOR TTrndx.Init(Adpth, Apscd, Anmcd : Byte);
BEGIN
  dpth  := Adpth;
  pscd  := Apscd;
  nmcd  := Anmcd;
END;
(**------------------------------------------------------------------------**)

DESTRUCTOR TTrndx.Done;
BEGIN
END;
(**------------------------------------------------------------------------**)

FUNCTION TTrndx.GiveDpth: Byte;
BEGIN
  givedpth := dpth;
END;
(**------------------------------------------------------------------------**)

FUNCTION TTrndx.GivePscd: Byte;
BEGIN
  givepscd := pscd;
END;
(**------------------------------------------------------------------------**)

FUNCTION TTrndx.GiveNmcd: Byte;
BEGIN
  givenmcd := nmcd;
END;
(**------------------------------------------------------------------------**)

PROCEDURE TTrndx.SetNmcd(Anmcd: Byte);
BEGIN
  nmcd := Anmcd;
END;

(**----------------------------------------------------------1993-/-2019----**)
(** NdInfoStr displays the available information from the node point as folows:
(,creation sequence number, ), ascending order number of the schema at that level,
[, treedepth, -, nmcd written as a binary string, =, nmcd written as a number value, ]
   eg maxdpth 7: dpth 7, draw-root 24 gives: (53) 24[7-1110100=116]
   eg maxdpth 7: dpth 4, in-between node 7 gives: (100) 7[4-0110=6]
   eg maxdpth 4: dpth 4, draw-root 7 gives: (16) 7[4-0110=6]
   eg maxdpth 4: dpth 2, in-between node 3 gives: (18) 3[2-01=1]
   eg maxdpth 4: dpth 0, in-between node 1 gives: (nb1) 1[0] Final          **)

FUNCTION TTrndx.NdInfoStr(mxdpth: word):String;
  var  prndpth : string[2];
       prnrank : string[3];
       prnpscd : string[3];
       prnnmcd : string[3];
BEGIN
 (* prepare the print strings *)
  Str(GiveRank(pscd,dpth):3,prnrank); { returns the rank order as a string,
                                      :3 is the amount of digits of the number}
  Str(Givedpth,prndpth);    // returns the depth as a string
  Str(Givepscd,prnpscd);    // returns the position_code bit order as a string
  Str(Givenmcd,prnnmcd);    // returns the num_code as a string

(* concat for result voor Blaise_86_UK FTree program *)
  if Givedpth = 0 then                                            // finale
    Result := Concat(prnrank,' Final')
  else
    Result := Concat(prnrank,' ');

(* the original concat for result *)
  //if Givedpth = 0 then // final
  //  Result := Concat('(nr',prnnmcd,')',prnrank,'[',prndpth,']',' Final.')
  //else
  //  Result := Concat('(',prnnmcd,')',prnrank,'[',prndpth,
  //                        '-',GiveBinstr(Givepscd,Givedpth),'=',prnpscd,']');
END;                                                       // TTrnd.NdInfoStr
(**-------------------------------------------------------------------------**)

END.

