{*-- STD_LOG.pas ---------------------------------------------- april-2003----*
 * Standard Tree-knoop Class definitie                                        *
 * developed by Rik Smit at 20 februari 1993 voor Turbo Pascal                *
 * converted to Lazarus FreePascal 1 maart 2019                               *
 * last update 27 mrt 2019, 4 apr 2019                                     *
 *----------------------------------------------------------------------------*}

unit STD_LOG;                                                   {dd april 2003}

interface

  var indent: byte;                                            { indent value }
      logon: boolean;
  const logfilename : string = 'Treelog.txt';
  FUNCTION  GiveEXEPath: string;
  FUNCTION  GiveParamPath(cnt: byte): String;
  procedure LogInit(Pathname:string; logflnm:string; var logfile: text;
                                              txt: widestring; OnLog: Boolean);
  procedure LogDone(var logfile: text);
  procedure Logtext(var logfile: text;VAR indent: byte;
                                               txt: Widestring; cntnwln: byte);
  procedure SetIndent(var logfile: text;VAR indent: byte; newpos: byte);
  procedure Login (var logfile: text;VAR indent: byte; txt: widestring);
  procedure Logout(var logfile: text;VAR indent: byte; txt: widestring);

var Logfile : TEXT;
    datapad : string;                           { path to datafiles, add 2003 }

implementation

  const indentstr : string = '  ';

  (**------------------------------------------------------------------------**)
  FUNCTION GiveEXEPath: string;
  var chrps,                                                  { char position }
      hldchrps : byte;                                   { hold char position }
      Exepad : string;                           { path to executable program }
  BEGIN
   (* Exepad gets the full path name (.exe inclusive) to the running program *)
    ExePad := paramstr(0);
   (* search for the location of the last \ in the path string dd 02-2000*)
    chrps := Pos('\',ExePad);
    hldchrps := 0;
    Repeat
      hldchrps := hldchrps+chrps;       { keeps track of \ in the path string }
      ExePad := copy(ExePad,(chrps+1),255);{copy the path from the last found \}
      chrps := Pos('\',ExePad);                                { locat next \ }
    Until chrps = 0;
   (* Give the ExePad the string including the last found \.
      So excluding the executable name.exe                  *)
    if hldchrps <> 0 then Exepad := Copy(paramstr(0),1,hldchrps)
    else Exepad := '';
    Result := Exepad;
  END;                                                            { GiveEXEPat }
  (**------------------------------------------------------------------------**)

  FUNCTION GiveParamPath(cnt: byte): String;
  BEGIN
   (* a ExePad has been specified and the requested number is correct *)
    if (paramcount > 0) AND (cnt <= paramcount) then             { dd okt 2004 }
   (* if so, then add the name to the globale variable ExePad *)
      GiveParamPath := GiveExePath + paramstr(cnt) +'\'
    else GiveParamPath := GiveExePath;
  END;                                                          { GiveDataPath }
  (**------------------------------------------------------------------------**)

  PROCEDURE LogInit(Pathname:string; logflnm:string;
                         var logfile: text; txt: widestring; OnLog: Boolean);
  BEGIN
    Logon := Onlog;
    if Logon then
    BEGIN
      indent := 0;
    (* give datapad the path to the dir of the started application *)
      if Pathname = '' then
        datapad := GiveParamPath(2)
      else
        datapad := Pathname;
      assign(logfile,datapad+Logflnm);
      rewrite(logfile);
      writeln(logfile,'LOGINIT');
      writeln(logfile,txt);
      writeln(logfile);
    END;
  END;
  (**------------------------------------------------------------------------**)

  PROCEDURE Login(var logfile: text;VAR indent:byte; txt:widestring);
    var step : byte;
  BEGIN
    if logon then
    BEGIN
      INC(indent);
      for step := 1 to indent do write(logfile,indentstr);
      writeln(logfile,'IN : ',txt);
    END;
  END;
  (**------------------------------------------------------------------------**)

  PROCEDURE Logout(var logfile: text;VAR indent:byte; txt: widestring);
    var step : byte;
  BEGIN
    if logon then
    BEGIN
      for step := 1 to indent do write(logfile,indentstr);
      writeln(logfile,'OUT: ',txt);
      DEC(indent);
    END;
  END;
  (**------------------------------------------------------------------------**)

  PROCEDURE LogDone(var logfile: text);
  BEGIN
    if logon then
    BEGIN
      writeln(logfile,'LOGDONE');
      Close(logfile);
    END;
  END;
(*----------------------------------------------------------------------------*)

    procedure Logtext(var logfile: text;VAR indent: byte; txt: widestring; cntnwln: byte);
      var step : byte;
    Begin
      if logon then
      BEGIN
        for step := 1 to indent do write(logfile,indentstr);
        writeln(logfile,txt);
        for step := 1 to cntnwln do writeln(logfile);
      end;
    end;
(*----------------------------------------------------------------------------*)

    procedure SetIndent(var logfile: text;VAR indent: byte; newpos: byte);
    Begin
      indent := newpos;
    end;

END.
