(**-- STDTREE.pas -----------------------------------------------1993--- ----*
* Standaard Tree-object definitie                                            *
* developed by Rik Smit on 20 februari 1993.                                 *
* last update  20 mei 1994.                                                  *
* Update Printtree jan 2020 for Blaise_86_UK: use of unicade chars           *
* Used by Lazarus 2.0.6 FTree.exe AND Delphi Rio 10.3.3 DFtree.exe           *
**-------------------------------------------------------------------------**)


(*6--Code 6: The 1993 Tree Index Object definition  Blaise_86_UK------------*)
Unit STDTREE;

{$DEFINE useLazChar}// self-defined compiler directive for the use of the logfile

interface

uses SysUtils,                                 { for intern IntToStr routine }
     STD_CTV,                      { standard Const, Type and Var defintions }
     StdNode;                                       { node/index definitions }

(*5--Code 5: The 1993 Tree Index Object definition  Blaise_86_UK------------*)
type PTrnd = ^TTrnd;                                      { tree-node pointer}
     TTrnd = RECORD                                  { tree-node record type }
       up    : PTrnd;                           { pointer to another Up node }
       dn    : PTrnd;                           { pointer to another Dn node }
       rt    : PTrnd;                           { pointer to another Rt node }
    { New }
       ndx   : Ptrndx                      { treeindex ptr, unit stdnode.pas }
     END;
(*-------------------------------------------------------------------------5*)

     PTr = ^ TTr;                                       { tree Object pointer}
     TTr = OBJECT                                         { tree Object type }
       Nds :PTrnd;                                             { nodes field }

       constructor Init;
       destructor Done; Virtual;

       procedure  Add(nwndx: PTrndx);
       procedure  MakeTree(ATrdpth: Byte);
       procedure  PrintTree;

       function   GiveTrdpth: Byte;
       function   GiveRt: Pointer;
       function   CntNds: Word;
     END;
(*-------------------------------------------------------------------------6*)
(* GRAPHIC LINE ELEMENTS; Unicode chars. Used bij Lazarus, not by Delphi *)
{$IFDEF useLazChar}
const vrt  : widechar = #9474; {?}
      hrz  : widechar = #9472; {?}
      hup  : widechar = #9496; {?}
      hdn  : widechar = #9488; {?}
      hvl  : widechar = #9500; {?}
      Space: widechar = #8192; { }
      dot  : widechar = #11776; {.}
{$ELSE}
const vrt : char = '?';
      hrz : char = '?';
      hup : char = '?';
      hdn : char = '?';
      hvl : char = '?';
      Space: char = ' ';
      dot : char = '.';
{$ENDIF}


var treestart : Ptrnd;
    LST : Text;

implementation
//Uses Printer;
(**  TTr  -----------------------------------------------------------------**)
{$IFDEF useLazChar}
  uses Ftree;   // Lazarus program
{$ELSE}
  uses DFtree;   // Delphi program
{$ENDIF}


CONSTRUCTOR TTr.Init;
BEGIN
  Nds := NIL;
END;
(**------------------------------------------------------------------------**)

DESTRUCTOR TTr.Done;

  PROCEDURE PostOrder(reflink : PTrnd);
  BEGIN
    if reflink <> NIL then
    BEGIN
      PostOrder(reflink^.Up);
      PostOrder(reflink^.Dn);
      Dispose(reflink^.ndx,Done);
      Dispose(reflink);
    END;
  END;

BEGIN
  PostOrder(treestart);
END;                                                              { TTr.Done }
(**------------------------------------------------------------------------**)

PROCEDURE TTr.Add(nwndx: PTrndx);
  var nwnd: PTrnd;
BEGIN
  New(nwnd);
  with nwnd^ do
  BEGIN
    ndx := nwndx;
    up := NIL;
    dn := NIL;
    rt := Nds;
    nds := nwnd;
  END;
END;                                                               { TTr.Add }
(**------------------------------------------------------------------------**)

PROCEDURE TTr.MakeTree(ATrdpth: Byte);
  var strtcnt: Byte;

  PROCEDURE PreOrder(reflink: PTrnd);
  BEGIN
    if Reflink^.ndx^.dpth < ATrdpth then
    BEGIN              // not yet at the requested depth: so make 2 new nodes
      with reflink^ do
      BEGIN
      (** save the reflink-Ptr for the new Up-node^.Rt in nds
          save then reflink link here for reuse beccause
          ...it immediately gets in AddNode a new value         **)
        nds := reflink;
      (** make the new Up root **)
        Add(New(PTrndx,Init(Succ(ndx^.dpth),ndx^.pscd,0)));
      (** give the new ptr to ^.UP **)
        Up  := nds;                                         { make the link }
      (** save the reflink-Ptr for the new Dn-node^.Rt in nds **)
        nds := reflink;
      (** make the new Dn root **)
        Add(New(PTrndx,Init(Succ(ndx^.dpth),
                                     SetBit(Succ(ndx^.dpth),ndx^.pscd),0)));
      (** give the new ptr to ^.DN **)
        Dn  := nds;                                         { make the link }
      END;
      PreOrder(Reflink^.Up);             { first make the top half,   }
      PreOrder(Reflink^.Dn);             { then the bootom half of the tree }
    END;
  END;

BEGIN
  Ttr.Init;                                                       { init nds }
  Add(New(PTrndx,Init(0,0,0)));                           { create root-node }
  treestart := nds;                         { treestart gets adres root-node }
  Preorder(nds);                                                { built tree }
END;                                                          { TTr.MakeTree }
(**------------------------------------------------------------------------**)
FUNCTION TTr.Givetrdpth: Byte;
  var maxdepth: Byte;

  PROCEDURE PreOrder(reflink: PTrnd);
  BEGIN
    if reflink <> Nil then
    BEGIN
      If reflink^.ndx^.Givedpth > maxdepth
        Then Inc(maxdepth);
      Preorder(reflink^.Up);
      Preorder(reflink^.Dn);
    END;
  END;

BEGIN
  maxdepth := 0;
  Preorder(treestart);
  givetrdpth := maxdepth;
END;                                                        { TTr.Givetrdpth }
(**------------------------------------------------------------------------**)
FUNCTION TTr.GiveRt : Pointer;
BEGIN
  GiveRt := Nds^.rt;
END;                                                            { TTr.GiveRt }
(**------------------------------------------------------------------------**)
FUNCTION TTr.CntNds: Word;
  var ndcnt : Word;

  PROCEDURE InOrder(reflink: PTrnd);
  BEGIN
    If reflink <> Nil Then
    BEGIN
      Inorder(reflink^.Up);
      Inc(ndcnt);
      Inorder(reflink^.Dn);
    END;
  END;

BEGIN
  ndcnt := 0;
  Inorder(treestart);
  CntNds := ndcnt;
END;                                                            { TTr.CntNds }
(**------------------------------------------------------------------------**)
//PrintTree: print a tree to a textfile
PROCEDURE TTr.PrintTree;

  type showstr = string[15]; // length printstr = standard distance between...
                             // ...the nodes at different tree depths.
  var  cnt,                  // node counter
       trdpth : Byte;        // tree depth
       s : showstr;
  (**------------------------------------------------------------------------**)
  PROCEDURE InOrder(Reflink: PTrnd; n: byte);
    var i       : byte;                                       { loop counter }
        intern  : Boolean;
        prnstr  : string;                            { text file printstring }
        mmstr   : string;                                      { memo-string }

  BEGIN
    If reflink <> Nil Then
    BEGIN
     // print upper part of the schedule tree
      Inorder(reflink^.Up,n-1);

      if reflink^.ndx^.Givedpth >= 0 then        { the breaks down condition }
      BEGIN
        SetLength(s,High(s));           { High(s)returns the max lengte of s }
        SetLength(prnstr,0);                                         { innit }
        SetLength(mmstr,0);                                          { innit }
        Fillchar(s[1],High(s),' ');   { fills entire show string with spaces }

(* Note: dpth is zero by the draw-root where the fields FUp and FDn are NIL ! *)

        for i := 1 to N do write(LST,s);{ print the correct number of s units =
                                          equivalent to moving the print cursor }
        for i := 1 to N do mmstr :=  mmstr + s;

        for i := 2 to N do write(LST,' ');{ extra spaces for the (N-1) number of
                                         graphic char(s) used in the line above }
        for i := 2 to N do mmstr :=  mmstr + ' ';

        if Reflink^.ndx^.Givedpth <> trdpth Then
        Begin
          write(LST,hvl); {?}
          mmstr := mmstr + hvl;                   { hvl = the graphic char ? }
        End;

      (* Get the description of this node *)
        s := Reflink^.ndx^.NdInfoStr(N);  { NodeInfoString(.,.) see STDNODE! }
        write(LST,s);
        mmstr := mmstr + s;

        if Reflink^.ndx^.Givedpth > 0 then             { exclude finale node }
          for i := 1 to (High(s)-Length(s)) do
          Begin
            write(LST,hrz);        { fill to standard length(20) with dashes }
            mmstr := mmstr + hrz;  {?}
          End;
        Inc(cnt);

        if (trdpth-n) <> 0 then
          With reflink^.ndx^ do
          BEGIN
// if a node at (Trdpth-n) level has a bit =1, then write graphic element {?}
            if BitSet((trdpth-n),Givepscd) then         { this is a Dn node }
            Begin
              write(LST,hup);
              mmstr := mmstr + hup; {?}
            End
// else: (Trdpth-n) level has a bit =0, so write here graphic element {?}
            else
              if NOT BitSet((trdpth-n),Givepscd) then   { this is a Up node }
              Begin
                write(LST,hdn);
                mmstr := mmstr + hdn; {?}
              End;

            intern := FALSE;
            SetLength(prnstr,0);
            SetLength (s,High(s));
            Fillchar(s[1],High(s),' ');

         (*  Question: have line elements to be placed further on as a rule? *)
            for i := {1}2 to Givedpth do
            BEGIN
   // Is there a sign change (0=>1 of 1=>0) at position i in the position-code?
              if (BitSet(i,Givepscd) <> BitSet(i-1,Givepscd)) then
             // concat prnstr after s with the graphical element ? in between
              BEGIN
                prnstr := Concat(s,vrt,prnstr); {?}{ place the prnstr behind s }
                intern := TRUE;
              END
              else                                          { no sign change }
            // concat prnstr after s with a space char instead of the ...
            // ...graphical element ? in between
                if intern then prnstr := Concat(s,' ',prnstr);
            END;  {for}

            write(LST,prnstr);
            mmstr := mmstr + prnstr;
          END;
      END;
      writeln(LST);
      form1.memo1.Lines.Append(mmstr);
     // print lower part of the schedule tree
      Inorder(reflink^.Dn,n-1);
    END;
  END;

BEGIN
//Innit parameters
  trdpth := Givetrdpth;                                    { give tree depth }
  cnt := 0;                                        { initiation node counter }
  form1.memo1.Text := ('Schema with depth '+IntToStr(trdpth));
  Inorder(treestart,trdpth);         { print entire tree structure recursive }
  write(LST,'Node count: ',IntToStr(cnt));
  form1.memo1.Lines.Append('Node count: '+IntToStr(cnt));
END;                                                         { TTr.PrintTree }
(**------------------------------------------------------------------------**)
BEGIN
  treestart := NIL;                                   { initiation treestart }
END.
