(*-- Unit DisplayForm.pas --------------------------------------15-04-2019---*
 * Print a tree on the Forms Canvas                                          *
 * Standard Lazarus 2.0.6 Tree Class definition                              *
 * Based on the 1993 DOS pascal program tree.pas by Rik Smit                 *
 * developed by Rik Smit  2019/2020                                          *
 * last update 27 mrt 2019, dec 2019                                         *
 *---------------------------------------------------------------------------*)

unit DisplayForm;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, StdCtrls;

type

  { Tfrmdspl }

  Tfrmdspl = class(TForm)
    ScrollBarH: TScrollBar;
    ScrollBarV: TScrollBar;
    procedure FormActivate(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure InnitScrollBars(Sender: TObject);
    procedure ScrollBarHScroll(Sender: TObject; ScrollCode: TScrollCode;
      var ScrollPos: Integer);
    procedure ScrollBarVScroll(Sender: TObject; ScrollCode: TScrollCode;
      var ScrollPos: Integer);
  private
    FmxRw,     // field max rows to print
    FRwHght,   // field row height
    FvsblRw,   // rows insite canvas area
    FunvsblRw, // rows outsite canvas area
    Fmxcl,     // field max collums to print
    FClwdth,   // field collom width
    Fvsblcl,   // colloms insite canvas area
    Funvsblcl: Integer;   // colloms outsite canvas area
    procedure  SetScrollBars;
  public

  end;

var
  frmdspl: Tfrmdspl;

implementation

{$R *.lfm}

uses UTrNdClass,utrclass,UTreeNode;

{ Tfrmdspl }

procedure Tfrmdspl.InnitScrollBars(Sender: TObject);
  var  ndcnt  : Word;      { count Nodes }
       trdpth : Byte;      { tree  depth }
       s : string[25];     { length printstr = space between the nodes on --}
                           { -- opeenvolgende tree diepten.                 }
                           { zou je eventueel variabel kunnen maken         }
       cntwrtln: integer;  { count used lines                               }
       ww,hh   : Integer;  { font wide/hight, hight used in Canvas method   }
       xps     : Integer;  { relative position x-axe                        }
       intern  : Boolean;  { valt 'binnen' treeline schema of daar 'buiten' }
       prnstr  : string;

(* Info: High(s) geeft max lengte van s = string[25] = 25 terug. Wordt overal
         gebruikt waar deze max lengte van s nodig is. Wil je de lengte van s
         aanpassen dan hoef je alleen s in de type definitie aan te passen:
         door gebruikt te maken van High(s) wordt dan in de code automatisch
         de juiste lengte gebruikt.                                          *)

(**- InOrder() is used recursively! -------------------------------------**)
  PROCEDURE InOrder(Reflink: TTrnd; n: integer);
    var i: integer;                                           { loop counter }

  BEGIN
    If reflink <> Nil Then                      { start at the deepest nodes }
    BEGIN
      Inorder(reflink.GetFUp,n-1);   { print upper part of the schedule tree }

      if reflink.GetFdpth >= 0 then
      BEGIN
        Fillchar(s[1],High(s),' ');                     { fill s with spaces }

        // Get length s (= ww)
        canvas.GetTextSize(s,ww,hh);

        for i := 1 to N do
          with Canvas do
         //write s
         // TextOut(((i-1)*ww) ,NdCnt*(hh),s);  // old: without scrollbar
        TextOut(PenPos.x ,NdCnt*(hh),s);

        for i := 2 to N do
          with canvas do
          //write Space
          TextOut((PenPos.x) ,NdCnt*(hh),' ');

        if Reflink.GetFdpth <> trdpth Then
          with canvas do
          //write hvl = ├
            TextOut((PenPos.x),NdCnt*(hh),hvl);  {├}

        s := Reflink.NdInfoStr; // NdInfoStr(...) find in UTrNdClass!
        // write s
        with canvas do TextOut((PenPos.x),NdCnt*(hh),s);
        if Reflink.GetFdpth > 0 then       { exclude finale node (Froot)}

      (* High(s) is max length of s = string[25] = 25
         Lenght(s) is length of s on this moment and depents of length NdIfoStr *)
          //write hrz = ─
          for i := 1 to (High(s)-Length(s)) do  { fill to standard length(20) with dashes {─} }
            with canvas do TextOut((PenPos.x),NdCnt*(hh),hrz);  {─}

        Inc(Ndcnt);                                            // count nodes

        if (trdpth-n) <> 0 then
          With reflink do
          BEGIN
         (* bitset: TRUE ==> if a node at (Trdpth-depth) level has a bit = 1,
                                              then write graphic element {┘} *)
            if BitSet((trdpth-n),GetFpscd) then
              // write up-hook char: hup
              With frmdspl.canvas do TextOut((PenPos.x),(NdCnt-1)*(hh),hup)  {┘}
         (* else:FALSE ==> (Trdpth-depth) level has a bit =0,
                                           so write here graphic element {┐} *)
            else
              // write down-hook char: hdn
              With frmdspl.canvas do TextOut((PenPos.x),(NdCnt-1)*(hh),hdn); {┐}

          (* 1) prepair prnstr *)
            intern := FALSE;                                         { innit }
            SetLength(prnstr,0);                                     { innit }
            s[0] := Chr(High(s) );          { High(s)returns max length of s }
            Fillchar(s[1],High(s),' ');  { fills entire s string with spaces }

(*  Question: have line elements to be placed further on in this line as a rule?
    That only applies within the tree structure and based on a switch
    (..01.. 0r ..10..) in the pscd string. Example: 00010010.                 *)
            for i := 2 to GetFdpth do
            BEGIN
(* Is there a sign change (0=>1 of 1=>0) at position i in the position-code? *)
              if (BitSet(i,GetFpscd) <> BitSet(i-1,GetFpscd)) then
              BEGIN
              // concat prnstr after s with the graphical element │ in between
                prnstr := Concat(s+vrt,prnstr);  { place the prnstr behind s }
              intern := TRUE;
              END
            { no sign change }
              else { if intern: concat prnstr after s with a space char  ...  }
                            {...instead of the graphical element │ in between }
                if intern then
                  prnstr := Concat(s+' ',prnstr);
            END;  { For.... }

          (* 2) write prnstr *)
            With frmdspl.canvas do
            Begin
              TextOut((PenPos.x),(cntwrtln)*(hh),prnstr);
            End;

          END;  { if (trdpth-n) <> 0 then With reflink do }
      END; { if reflink.Givedpth >= 0 }

      // writeln
      Inc(cntwrtln);
      canvas.MoveTo((xps),cntwrtln+1);            { back to pos(xy)= (0,y+1) }

      Inorder(reflink.GetFdn,n-1);   { print lower part of the schedule tree }

    END; { If reflink <> Nil }
  END; { Inorder() }

BEGIN  { printtree }
  trdpth := tree.Givetrdpth;

  if FmxRw > FvsblRw  then
  Begin
    ScrollBarV.Visible:= True;
    //text := ' Ver. position = '+inttostr(ScrollBarV.Position);
  end
  else
  Begin
    ScrollBarV.Visible:= False;
    //text := '';
  end;

  if FmxCl > FvsblCl  then
  Begin
    ScrollBarH.Visible:= True;
    //text := ' Hor. position = '+inttostr(ScrollBarH.Position);
  end
  else
  Begin
    ScrollBarH.Visible:= False;
    //text := '';
  end;

  canvas.clear;

// Innit Scrolling pos X and Y
  Ndcnt    := 0 - ScrollBarV.position;
  cntwrtln := 0 - ScrollBarV.position;
  xps      := 0 - ScrollBarH.position*FClwdth;
  Canvas.MoveTo(xps,cntwrtln);

  Inorder(tree.GetFroot,trdpth);  // start tree building at Tree.Froot

//at the end display node count: Ndcnt
  With frmdspl.canvas do
  Begin
    MoveTo((xps),(cntwrtln*hh)+1);   // back to pos(xy)= (0,y+1)
    TextOut((PenPos.x),PenPos.y,'Nodes count is: '+IntToStr(Tree.CntNds));
  End;

END;                                                         { TTr.PrintTree }

procedure Tfrmdspl.ScrollBarHScroll(Sender: TObject; ScrollCode: TScrollCode;
  var ScrollPos: Integer);
begin
  // innit ScrollbarHorizontaal
  with ScrollBarH do
  begin
   FvsblCl := canvas.Width div FClwdth;
   if (Max > FvsblCl) then
   Begin
     Visible := True;
     FUnvsblCl := Max - FvsblCl;
     if FunvsblCl < FvsblCl  then
     begin
       LargeChange := Round((1-(FUnvsblCl/FvsblCl))* max);
     end
     else
     begin
       LargeChange := FvsblCl;
     end;
     PageSize := LargeChange;
   End {if ()}
   else Visible := False;
end; { with}

end;

procedure Tfrmdspl.ScrollBarVScroll(Sender: TObject; ScrollCode: TScrollCode;
  var ScrollPos: Integer);
begin
 with ScrollBarV do
 Begin
 // Innit Row Fields
  if FrwHght > 0 then
    FvsblRw := canvas.Height div FrwHght // def at formcreate
  else FvsblRw := canvas.Height div 20;   // standaard choise

  if (Max > Fvsblrw) then
  Begin
    Visible := True;
    FunvsblRw := Max - FvsblRw;
    if FUnvsblrw < FvsblRw then
    begin
      LargeChange := Round((1-(FUnvsblRw/FvsblRw))* max);
    end
    else
    begin
      LargeChange := FvsblRw;
    end;
    PageSize := LargeChange;
  End {if ()}
  else Visible := False;
 end; {with }

end;

procedure Tfrmdspl.FormCreate(Sender: TObject);
begin
  // innit Canvas
  With Canvas do
  Begin
    Font.Name := 'Courier New';
    Font.Size := 8;
    Font.Color := clBlack;   //text Color
    Brush.Style:= bsClear;   //background Color
    Brush.Color:= clDefault; //clwhite;
    clear;
   // innit Field line height and col width
    FrwHght := GetTextHeight('H'); // = 16
    Fclwdth := GetTextWidth('H');  // = 8 = font size = char length
    FunvsblRw := 0 ;
    FunvsblCl := 0 ;
  end;
end;

procedure Tfrmdspl.FormActivate(Sender: TObject);
begin
 with ScrollbarV do
 begin
   SetParams(0,0,(tree.cntnds+1));  // (position, min, max)
   FmxRw := Max;
 end;
 with ScrollbarH do
 begin
   SetParams(0,0,(tree.GiveTrdpth*26)+20);  // (position, min, max)
   FmxCl := Max;
 end;

 SetScrollBars;
end;

procedure Tfrmdspl.SetScrollBars;
Begin
  with ScrollBarV do
  Begin
  // Innit Row Fields
   if FrwHght > 0 then
     FvsblRw := canvas.Height div FrwHght // def at formcreate
   else FvsblRw := canvas.Height div 20;   // standard choise

   if (Max > Fvsblrw) then
   Begin
     Visible := True;
     FunvsblRw := Max - FvsblRw;
     if FUnvsblrw < FvsblRw then
     begin
       LargeChange := Round((1-(FUnvsblRw/FvsblRw))* max);
     end
     else
     begin
       LargeChange := FvsblRw;
     end;
     PageSize := LargeChange;
   End {if ()}
   else Visible := False;

  end;

  // innit ScrollbarHorizontaal
  with ScrollBarH do
  begin
   FvsblCl := canvas.Width div FClwdth;
   if (Max > FvsblCl) then
   Begin
     Visible := True;
     FUnvsblCl := Max - FvsblCl;
     if FunvsblCl < FvsblCl  then
     begin
       LargeChange := Round((1-(FUnvsblCl/FvsblCl))* max);
     end
     else
     begin
       LargeChange := FvsblCl;
     end;
     PageSize := LargeChange;
   End {if ()}
   else Visible := False;

  end; {with }

end;

end.





