{*-- utrclass.pas ----------------------------------------------30-01-2019----*
 * Standard Lazarus 2.0.6 Tree Class definition                               *
 * Based on the 1993 DOS pascal program tree.pas by Rik Smit                  *
 * developed by Rik Smit  2019/2020                                           *
 * last update 27 mrt 2019, dec 2019                                          *
 *----------------------------------------------------------------------------*}

unit utrclass;

{$mode objfpc}{$H+}

interface

uses
  SysUtils, UTrNdClass,lazUTF8;

type

  TTr = class(TObject)
  private
    Froot: TTrNd; // Tree start point
  public
    // place any public fields here
    constructor Create;
    destructor  Destroy; virtual;
    destructor  LogDestroy;

    procedure  AddNode(Adpth, Apscd, Anmcd : Byte; var RootNode: TTrNd);
    function   MakeTree(ATrdpth: Byte): TTrNd;
    function   GiveTrdpth: Byte;
    function   GiveRt: Pointer;
    function   CntNds: Word;

    procedure   SetFroot(ANode: TTrNd);
    function    GetFroot: TTrNd;
    procedure   PrintTree;
    Procedure   PrnLogTr;
  end;

// const treeline draw elements of type unicode widechar

const vrt : widechar = #9474; {│}
      hrz : widechar = #9472; {─}
      hup : widechar = #9496; {┘}
      hdn : widechar = #9488; {┐}
      hvl : widechar = #9500; {├}


var LST : Textfile;

implementation

uses STD_LOG;        // logfile definitions

{$DEFINE usedebug}  //self-defined compiler directive for the use of the logfile

{ Global var's for tracking memmory use /itemno; see heaph.inc file }
var
  memrest: QWord;
  memstr : string;

(**  TTr  Class definitions -----------------------------------------------**)

CONSTRUCTOR TTr.Create;
BEGIN
  Inherited;
  Froot := NIl;
END;
(**------------------------------------------------------------------------**)

{*-- TTr.Destroy -----------------------------------------------30-04-2019----*
 * Destructor for tree's nodes                                                *
 * developed 20 februari 1993                                                 *
 *----------------------------------------------------------------------------*}
DESTRUCTOR TTr.Destroy;

  PROCEDURE PostOrder(reflink : TTrnd);
  BEGIN
    if reflink <> NIL then
    BEGIN
      PostOrder(reflink.GetFUp);
      PostOrder(reflink.GetFDn);
     // both end nodes are NIL so they can be cleaned up
      reflink.Free;
    END;
  END;

BEGIN
  PostOrder(GetFRoot);
  inherited;
END;                                                             { TTr.Destroy }
  (**------------------------------------------------------------------------**)

{*-- TTr.LogDestroy --------------------------------------------30-04-2019----*
 * Destructor for tree's                                                      *
 * developed 20 februari 1993                                                 *
 * last update mei 2019: add logfile                                          *
 *----------------------------------------------------------------------------*}
DESTRUCTOR TTr.LogDestroy;
var Logfile : TEXT;
    ndcnt   : word;   // node counter

  PROCEDURE PostOrder(reflink : TTrnd);
  BEGIN
    if reflink <> NIL then
    BEGIN
     {$IFDEF usedebug}
//      LOGIN(logfile,indent,'Reflink.GetFUp.'+reflink.NdInfoStr(reflink.getFdpth));
     LOGIN(logfile,indent,'Reflink.GetFUp.'+reflink.NdInfoStr);
     {$ENDIF};

      PostOrder(reflink.GetFUp);

     {$IFDEF usedebug}
      LOGText(logfile,indent,'> max depth: FUp node = NIL',0);
      LOGOUT(logfile,indent,'Reflink.GetFUp');

      LOGIN(logfile,indent,'Reflink.GetFDn.'+reflink.NdInfoStr);
     {$ENDIF};

      PostOrder(reflink.GetFDn);

     {$IFDEF usedebug}
      LOGText(logfile,indent,'> max depth: FDn node = NIL',0);
      LOGOUT(logfile,indent,'reflink.GetFDn');

      LOGIN(logfile,indent,'Reflink.Free: '+reflink.NdInfoStr);
      Inc(ndcnt);
     {$ENDIF};

      reflink.Free;

     {$IFDEF usedebug}
      LOGText(logfile,indent,'Deleted node cnt is: '+inttostr(ndcnt),0);
      LOGOUT(logfile,indent,'Reflink.Free');
     {$ENDIF};
    END;

  END;

BEGIN
 {$IFDEF usedebug}
  LOGINIT(datapad,'LOG Destroy-Tree.txt', Logfile,'clean up Tree Nodes', true);
  LOGIn(logfile,indent,'Start TTr.Destructor Distroy');
// number of Cntnds should ultimately equal the number of removed nodes
  LOGText(logfile,indent,'Total Nodes in this tree: '+inttostr(CntNds),1);
  ndcnt := 0; // reset counting var
  LOGText(logfile,indent,'Start cnt: '+inttostr(ndcnt),1);
 {$ENDIF};

//  PostOrder(treestart);
  PostOrder(GetFRoot);

 {$IFDEF usedebug}
  LOGOUT(logfile,indent,'Finish Destroy');
  LOGText(logfile,indent,'',1);
  LOGText(logfile,indent,'Total deleted nodes is '+inttostr(ndcnt),1);
  LOGDONE(logfile);
 {$ENDIF};
  inherited;
END;                                                          { TTr.LogDestroy }
(**------------------------------------------------------------------------**)
// Note the use of the word var for the RootNode variable
PROCEDURE TTr.AddNode(Adpth, Apscd, Anmcd : Byte; var RootNode: TTrNd);
 var Newnode: TTrNd;
BEGIN
  NewNode := TTrNd.Create(Adpth, Apscd, Anmcd);         // instantiate NewNode
  NewNode.SetFrt(RootNode);   // give the field newNode.Frt the RootNode value
  RootNode := NewNode;             // connect the new node to the calling node
END;                                                               { TTr.Add }
(**------------------------------------------------------------------------**)

(** MakeTree is a function that returns the beginning of the tree
    via the address of FRoot.                                               **)
FUNCTION TTr.MakeTree(ATrdpth: Byte):TTrNd;
Var prnrank : string[3];    { print tree ranking (from the top down )}
    prndpth : string[1];    { print depth }
    ndcnt   : word;         { count nodes }
    prncnt  : string[3];    { print node counter }
(* Logfile Local var's for tracking memmory use /itemno see heaph.inc file *)
    memstart,
    memstop      : QWord;
    myheapstatus : TFPCHeapStatus;
    Logfile      : TEXT;
    treestart    : TTrNd;

  PROCEDURE PreOrder(reflink: TTrNd);
    var tempnd : TTrNd;
  BEGIN
    if Reflink.GetFdpth < ATrdpth then//not yet at requested depth: make two new nodes
    BEGIN
      with reflink do
      BEGIN
       (** save the reflink-Ptr (Frt-link) link for the new FUp/FDn-node^.Rt
                                                           in the var tempnd **)
        tempnd := reflink;   //  save then reflink link here for reuse because…
                             //  ...it immediately gets in AddNode a new value
       (** make the new Up root  **)
        Inc(ndcnt);                                    // increase knots counter
        AddNode(Succ(GetFdpth), GetFpscd, ndcnt,tempnd);
       (** give the new ptr to ^.UP **)
        SetFUp(tempnd);                               // make the link
       (** make the new Dn root  **)
        tempnd := reflink;                            // restore the tempnd link
        Inc(ndcnt);                                   // increase knots counter
        AddNode(Succ(GetFdpth),SetBit(Succ(GetFdpth),GetFpscd),ndcnt,tempnd);
       (** give the new ptr to ^.DN **)
        SetFDn(tempnd);                                      // make the link

       {$IFDEF usedebug}
        LOGText(logfile,indent,'Add(new UP node):     Str = '+GetFup.NdInfoStr,0);
        LOGText(logfile,indent,'....new Up.Root node: Str = '+GetFup.GetFrt.NdInfoStr,0);

        LOGText(logfile,indent,'Add(new DN node):     Str = '+GetFDn.NdInfoStr,0);
        LOGText(logfile,indent,'....new Dn.Root node: Str = '+GetFdn.GetFrt.NdInfoStr,0);
        If NdCnt <= 4 then  // show only t athe first time
        Begin
          LOGText(logfile,indent,'Thus: TreeStart.Up: Str = '+
                                      GetFroot.GetFup.NdInfoStr,0);
                                  //     treestart.GetFup.NdInfoStr(Atrdpth),0);
          LOGText(logfile,indent,'Thus: TreeStart.Dn: Str = '+
                                       GetFroot.GetFdn.NdInfoStr,0);
        End;

        LOGText(logfile,indent,'',0);
       {$ENDIF};
      END;

      {$IFDEF usedebug}
      With Reflink.GetFUp do
      Begin
        Str(GiveRank(GetFpscd,GetFdpth),prnrank);
        Str(GetFdpth,prndpth);
        Str(GetFnmcd,prncnt);
        LOGIN(logfile,indent,'(Nr '+prncnt+') = R'+prnrank+'/D'+prndpth+
                                                    '. Preorder(Reflink.FUp)');
      End;
      {$ENDIF};
   (* Make the top half of the tree first, *)
      PreOrder(Reflink.GetFUp);

      {$IFDEF usedebug}
      With Reflink.GetFUp do
      Begin
        if GetFdpth >= ATrdpth then
          LOGText(logfile,indent,'  ==> max depth: No new Up node needed',0);
        Str(GiveRank(GetFpscd,GetFdpth),prnrank);
        Str(GetFdpth,prndpth);
        Str(GetFnmcd,prncnt);
        LOGOUT(logfile,indent,'(Nr '+prncnt+') = R'+prnrank+'/D'+prndpth+
                                                    '. Preorder(Reflink.FUp)');
        LOGText(logfile,indent,'',0);
      End;

      With Reflink.GetFDn do
      Begin
        Str(GiveRank(GetFpscd,GetFdpth),prnrank);
        Str(GetFdpth,prndpth);
        Str(GetFnmcd,prncnt);
        LOGIN(logfile,indent,'(Nr '+prncnt+') = R'+prnrank+'/D'+prndpth+
                                                    '. Preorder(Reflink.FUp)');
      End;
     {$ENDIF};
   (* then the bottom half of the tree. *)
      PreOrder(Reflink.GetFDn);

      {$IFDEF usedebug}
      With Reflink.GetFDn do
      Begin
        if GetFdpth >= ATrdpth then
          LOGText(logfile,indent,'  ==> max depth: No new Dn node needed',0);
        Str(GiveRank(GetFpscd,GetFdpth),prnrank);
        Str(GetFdpth,prndpth);
        Str(GetFnmcd,prncnt);
        LOGOUT(logfile,indent,'(Nr '+prncnt+') = R'+prnrank+'/D'+prndpth+
                                                     '. Preorder(Reflink.FUp)');
        LOGText(logfile,indent,'',0);
      End;
     {$ENDIF};
    END;
  END;                                              { PreOrder(reflink: TTrNd)}

BEGIN
  {$IFDEF usedebug}
  LOGINIT(datapad,'LOG-MakeTree.txt', Logfile,'MakeTree', true);
  myheapstatus := GetFPCHeapStatus;
  memstart :=  myheapstatus.CurrHeapUsed;
  memstr := IntToStr(memstart);
  LOGText(logfile,indent,'Heap memory at start MakeTree: '+memstr,0);
  {$ENDIF};

  NdCnt := 1;
  TreeStart := nil;                                        { innit treestart }
  AddNode(0,0,Ndcnt,treestart);          { create 1ste 'ROOT' treestart-node }
  SetFRoot(treestart);                              { Froot becomes 1ste node}
  Result := GetFroot;           { return root node adress as function result }

  {$IFDEF usedebug}
  LOGText(logfile,indent,'',0);
  LOGText(logfile,indent,'Start MakeTree',1);
  With treestart do
  Begin
    Str(GiveRank(GetFpscd,GetFdpth),prnrank);
    Str(GetFdpth,prndpth);
    Str(GetFNmcd,Prncnt);                      { print node production order }
    LOGText(logfile,indent,'Add Start Node: (0,0,1,TreeStart)',0);
    LOGText(logfile,indent,'Tree.Froot := TreeStart',0);
    LOGIN(logfile,indent,'(Nr '+prncnt+') = R'+prnrank+'/D'+prndpth+
                     '. Preorder(TreeStart): str = '+treestart.NdInfoStr);
  End;
  {$ENDIF};

(* built tree with Froot as startpoint *)
  Preorder(TreeStart);

  {$IFDEF usedebug}
  With treestart do
  Begin
    Str(GiveRank(GetFpscd,GetFdpth),prnrank);
    Str(GetFdpth,prndpth);
    Str(GetFNmcd,Prncnt);                      { print node production order }
    LOGOUT(logfile,indent,'End Nr '+prncnt+' = R'+prnrank+'/D'+prndpth+
                      '. Preorder(TreeStart): str = '+treestart.NdInfoStr);
    LOGText(logfile,indent,'Result := GetFroot: str = '+GetFroot.NdInfoStr,0);                                    //lege regelomslag
    LOGText(logfile,indent,'',1);                               { empty line }
  End;

  LOGText(logfile,indent,'Node count is: '+IntToStr(ndcnt),1);

  myheapstatus := GetFPCHeapStatus;
  memstop :=  myheapstatus.CurrHeapUsed;
  memrest := (memstop-memstart);
  memstr := IntToStr(memrest);
  LOGOUT(logfile,indent,'Memory used at end MakeTree: '+memstr);
  LOGDONE(Logfile);
  {$ENDIF};
END;                                                          { TTr.MakeTree }
(**------------------------------------------------------------------------**)

FUNCTION TTr.Givetrdpth: Byte;

  PROCEDURE PreOrder(reflink: TTrNd);
  BEGIN
    if reflink <> Nil then
    BEGIN
      If reflink.GetFdpth > Result Then
        Inc(Result);
      Preorder(reflink.GetFUp);
      Preorder(reflink.GetFDn);
    END;
  END;

BEGIN
  Result := 0;
  Preorder(GetFRoot);
END;                                                        { TTr.Givetrdpth }
(**------------------------------------------------------------------------**)
{ TTr.GiveRt }
FUNCTION TTr.GiveRt : Pointer;
BEGIN
  GiveRt := Froot;
END;
(**------------------------------------------------------------------------**)

FUNCTION TTr.CntNds: Word;

  PROCEDURE InOrder(reflink: TTrNd);
  BEGIN
    If reflink <> Nil Then
    BEGIN
      Inorder(reflink.GetFUp);
      Inc(Result);
      Inorder(reflink.GetFDn);
    END;
  END;

BEGIN
  Result := 0;
  Inorder(GetFRoot);
END;                                                            { TTr.CntNds }
(**------------------------------------------------------------------------**)

PROCEDURE TTr.SetFroot (ANode: TTrNd);
BEGIN
  FRoot := ANode;
END;
(**------------------------------------------------------------------------**)

FUNCTION TTr.GetFroot : TTrNd;
begin
  result := Froot;
end;
(**------------------------------------------------------------------------**)

(** PrintTree: print a tree to a textfile
**)
PROCEDURE TTr.PrintTree;
 (* const vrt : char = '|';      { vertikale verbindings streep       }
          hrz : char = '-';      { horinzontale verbindings streep    }
          hup : char = '/';      { haakje up                          }
          hdn : char = '\';      { haakje dn                          }
          hvl : widechar = '>';  { verbinding horizontaal vertikaal   }

 const vrt : widechar = #9474; {│}
       hrz : widechar = #9472; {─}
       hup : widechar = #9496; {┘}
       hdn : widechar = #9488; {┐}
       hvl : widechar = #9500; {┐}
*)
  var  cntnd  : word;     { nodes counter }
       trdpth : Byte;     { tree depth}
       s : string[25];    { length printstr = = standard distance between...}
                          { ...the nodes at different tree depths.          }

  (**- InOrder() is used recursively! -------------------------------------**)
  PROCEDURE InOrder(Reflink: TTrnd; depth: integer);
    var i       : byte;                                       { loop counter }
        intern  : Boolean;              {falls within or outside the schedule}
        prnstr  : string;                                      { print string}
  BEGIN
    If reflink <> Nil Then
    BEGIN

      Inorder(reflink.GetFUp,depth-1); { print upper part of the schedule tree }

     (* Note: dpth is zero by the draw-root where FUp and FDn are NIL ! *)
      if reflink.GetFdpth >= 0 then
      BEGIN
        for i := 1 to depth do write(LST,s);  // print the correct number of s
                                // units=equivalent to moving the print cursor

        for i := 2 to depth do write(LST,' ');   // extra spaces for the (N-1)
                           // number of graphic char(s) used in the line above

        if Reflink.GetFdpth <> trdpth Then write(LST,hvl); {├} // see info:

      (* Get the description of this node *)
        s := Reflink.NdInfoStr;             { NdInfoStr(.,.) see UTrNdClass! }
        write(LST,s);
        if Reflink.GetFdpth > 0 then            { exclude finale node (Froot)}
          for i := 1 to (High(s)-Length(s)) do
            write(LST,hrz);    { fill to standard length(20) with dashes {─} }
        Inc(cntnd);

        if (trdpth-depth) <> 0 then
          With reflink do
          BEGIN

(* if a node at (Trdpth-depth) level has a bit =1, then write graphic element {┘} *)

            if BitSet((trdpth-depth),GetFpscd) then      { this is a Dn node }
              write(LST,hup) {┘}
(* else: (Trdpth-depth) level has a bit =0, so write here graphic element {┐} *)

            else write(LST,hdn); {┐}                     { this is a Up node }

            intern := FALSE;                                          { innit }
            SetLength(prnstr,0);                                      { innit }
            s[0] := Chr(High(s) );           { High(s)returns max length of s }
            Fillchar(s[1],High(s),' ');    { fills entire s string with spaces }

(*  Question: have line elements to be placed further on in this line as a rule?
    That only applies within the tree structure and based on a switch
    (..01.. 0r ..10..) in the pscd string. Example: 00010010.                 *)

            for i := 2 to GetFdpth do
            BEGIN
 // Is there a sign change (0=>1 of 1=>0) at position i in the position-code?
              if (BitSet(i,GetFpscd) <> BitSet(i-1,GetFpscd)) then
              BEGIN
              // concat prnstr after s with the graphical element │ in between
                prnstr := Concat(s+vrt,prnstr);  { place the prnstr behind s }
                intern := TRUE;
              END
           { no sign change }
              else { if intern: concat prnstr after s with a space char  ...  }
                            {...instead of the graphical element │ in between }
                if intern then prnstr := Concat(s+' ',prnstr);
            END;  { For.... }

            if intern then write(LST,prnstr);

          END;  { if (trdpth-n) <> 0 then With reflink do }

      END; { if reflink.Givedpth >= 0 }

      writeln(LST);                                             // new line

      Inorder(reflink.GetFdn,depth-1);{ print lower part of the schedule tree }

    END; { If reflink <> Nil }
  END; { Inorder() }

BEGIN  { printtree }
(* Innit parameters *)
  trdpth := Givetrdpth;    // give tree depth
  cntnd := 0;               // initiation node counter
// s wordt bij heet eerste knooppunt nog niet gebruikt:
// daarna wordt het verderop in de code alsnog geïnitialiseerd
//  s := '';
//  Fillchar(s[1],High(s),'.');         { fills entire showstring with spaces }
  Inorder(GetFRoot,trdpth);     { print the entire tree structure recursively }

  write(LST,'Nodes count: ',cntnd);
END;                                                          { TTr.Printtree}
(**------------------------------------------------------------------------**)

(** PrnLogTr: (PRINT LOG TREE) print a tree to a textfile
**)

PROCEDURE TTr.PrnLogTr;
// lijn elementen van unicode widechar
  const// vrt : widechar = #9474; {│}  //#9475; //'┃';
       // hrz : widechar = #9472; {─}  //#9473; //'━';
        hup : widechar = #9496; {┘}  //#9499; //'┛';
        hdn : widechar = #9488; {┐}  //#9491; //'┓';
       // hvl : widechar = #9500; {├}  //#9507; //'┣';

// vervangende char karakters voor lijn elementen
  const vrt : char = '|';    { vertikale verbindings streep       }
        hrz : char = '-';      { horinzontale verbindings streep    }
      //  hup : char = '/';      { haakje up                          }
      //  hdn : char = '\';      { haakje dn                          }
        hvl : widechar = '>';  { verbinding horizontaal vertikaal   }

  var  cntnd  : word;     { nodes counter }
       trdpth : Byte;     { tree depth}
       s : string[25];    { length printstr = = standard distance between...}
                          { ...the nodes at different tree depths.          }

       holds : widestring;
       logfile : Text;

   (**- InOrder() is used recursively! -------------------------------------**)

  PROCEDURE InOrder(Reflink: TTrnd; depth: integer);
    var i       : byte;                                          { lusteller }
        intern  : Boolean;              {falls within or outside the schedule}
        prnstr  : string;                                      { print string}
        prnnmcd : string[3];                   { print string for numbercode }
  BEGIN
    If reflink <> Nil Then
    BEGIN
     {$IFDEF usedebug}
      Str(reflink.GetFnmcd:3,prnnmcd); { put the creation sequencenumber in prnnmcd }
      LOGIN(logfile,indent,prnnmcd+' = Inorder(reflink.GetFUP, depth: '+
                                                   IntToStr(trdpth-depth)+')');
     {$ENDIF};

      Inorder(reflink.GetFUp,depth-1);{ print upper part of the schedule tree }

     {$IFDEF usedebug}
      Str(reflink.GetFnmcd:3,prnnmcd);{ put the creation sequencenumber in prnnmcd }
      LOGOUT(logfile,indent,prnnmcd+' = Inorder(reflink.GetFUP, depth: '+
                                                   IntToStr(trdpth-depth)+')');
      LOGText(logfile,indent,'',0);
      LOGText(logfile,indent,'now collect the date from the node -'+prnnmcd+
                                                        ' - itself:',1);
     {$ENDIF};

     (* Note: dpth is zero by the draw-root where FUp and FDn are NIL ! *)
      if reflink.GetFdpth >= 0 then
      BEGIN

       {$IFDEF usedebug}
        if depth > 0 then
          LOGIN(logfile,indent,' for i := 1 to '+IntToStr(depth)+
                                            ' (= [trdpth-depth]) do (write s)');
       {$ENDIF};

        for i := 1 to depth do  // print the correct number of s
        Begin                   // units=equivalent to moving the print cursor
          write(LST,s);

         {$IFDEF usedebug}
          Holds := concat(holds,s); // place S in Holds for logging
          Inc(indent);
          LOGText(logfile,indent,'i='+IntToStr(i)+') write S: "'+s+'"',0);
          Dec(indent);
         {$ENDIF};
        End;

       {$IFDEF usedebug}
        if depth > 0 then
        Begin
          LOGOUT(logfile,indent,' for i := 1 to '+IntToStr(depth)+' do  write s');
          LOGText(logfile,indent,'(so far printed line => "'+holds+'")',0);
        End;
        if depth > 1 then
          LOGIN(logfile,indent,' for i := 2 to '+IntToStr(depth)+
                                       ' (= [trdpth-depth]) do (write spatie)');
       {$ENDIF};

        for i := 2 to depth do   // extra spaces for the (N-1)
                           // number of graphic char(s) used in the line above
        Begin
    //      write(LST,' ');  // normal
          write(LST,'+');  { for logging: place the + char instead of a space }

         {$IFDEF usedebug}
    //      Holds := concat(holds,' ');   // normal
          Holds := concat(holds,'+'); { place the + char in Holds for logging }
          Inc(indent);
    //      LOGText(logfile,indent,'i='+IntToStr(i)+') write Space: " "',0); // normal
          LOGText(logfile,indent,'i='+IntToStr(i)+') write Space: "+"',0);
          Dec(indent);
         {$ENDIF};
        End;

       {$IFDEF usedebug}
        if depth > 1 then
        Begin
          LOGOUT(logfile,indent,' for i := 2 to '+IntToStr(depth)+' do  write space');
          LOGText(logfile,indent,'(so far printed line => "'+holds+'")',0);
        End;
       {$ENDIF};

        if Reflink.GetFdpth <> trdpth Then
        Begin
          write(LST,hvl);

         {$IFDEF usedebug}
          Holds := concat(holds,hvl); { save hvl for logprint purposes }
          LOGText(logfile,indent,'GetFdpth <> trdpth ==> write "'+hvl+'"',0);
          LOGText(logfile,indent,'(so far printed line => "'+holds+'")',0);
         {$ENDIF};
        end;
      (* Get the description of this node *)
        s := Reflink.NdInfoStr; //NdInfoStr(.,.) see UTrNdClass!
        write(LST,s);
        if Reflink.GetFdpth > 0 then             { exclude finale node (Froot)}
          for i := 1 to (High(s)-Length(s)) do
            write(LST,hrz);     { fill to standard length(20) with dashes {─} }
        Inc(cntnd);

       {$IFDEF usedebug}
        Holds := concat(holds,s);              { save s for logprint purposes }
        LOGText(logfile,indent,'write node nr '+ IntToSTR(cntnd)+
                                                        ' in S'+': "'+s+'"',0);
       { this is how s will look }
        LOGText(logfile,indent,'(so far printed line => "'+holds+'")',0);

        if Reflink.GetFdpth > 0 then             { exclude finale node (Froot)}
        begin
          LOGText(logfile,indent,'No final Node, so:',0);
     Inc(indent);
          LOGText(logfile,indent,'for i := 1 to '+ IntToStr(High(s)-Length(s))+
                                                   ' (= rest unused S), do',0);
     Inc(indent);
          for i := 1 to (High(s)-Length(s)) do
          begin
            holds := Concat(holds,'-');      { save '-' for logprint purposes }
            LOGText(logfile,indent,'i='+IntToStr(i)+') write '+hrz ,0);
          end;
     Dec(indent,2);
        { this is how s will look }
          LOGText(logfile,indent,'(so far printed line => "'+holds+'")',0);
        end;
       {$ENDIF};

        if (trdpth-depth) <> 0 then
          With reflink do
          BEGIN
           {$IFDEF usedebug}
            LOGText(logfile,indent,'(trdpth-depth)=('+ IntToStr(trdpth)+'-'+
                                        IntToStr(depth)+') <> 0 ==> check:',0);
       Inc(indent);
           {$ENDIF};

 (* if a node at (Trdpth-depth) level has a bit =1, then write graphic element {┘} *)
            if BitSet((trdpth-depth),GetFpscd) then
            Begin
              write(LST,hup);  // {┘}
             {$IFDEF usedebug}
              holds := Concat(holds,hup);
              LOGText(logfile,indent,'BitSet => write "'+hup+'"',0);
              LOGText(logfile,indent,'(so far printed line => "'+holds+'")',0);
             {$ENDIF};
            End
            else
 (* else: (Trdpth-depth) level has a bit =0, so write here graphic element {┐} *)
            Begin
              write(LST,hdn); //{┐}
             {$IFDEF usedebug}
              holds := Concat(holds,hdn);
              LOGText(logfile,indent,'NOT BitSet => write "'+hdn+'"',0);
              LOGText(logfile,indent,'(so far printed line => "'+holds+'")',0);
             {$ENDIF};
            End;


            intern := FALSE;                                          { innit }
            SetLength(prnstr,0);                                      { innit }
            s[0] := Chr(High(s) );           { High(s)returns max length of s }
            Fillchar(s[1],High(s),' ');    { fills entire s string with spaces }

           {$IFDEF usedebug}
            LOGText(logfile,indent,'',0);
            LOGText(logfile,indent,'INNIT: Flag Intern := FALSE; and prnstr := "'+'";',0);
            LOGText(logfile,indent,'INNIT: S := "'+s+'";',1);
       Dec(indent);
            LOGIN(logfile,indent,' for i := 2 to '+IntToStr(GetFdpth)+
                            ' do (check for more line elements on this line)');
       Inc(indent);
           {$ENDIF};

 (*  Question: have line elements to be placed further on in this line as a rule?
     That only applies within the tree structure and based on a switch
     (..01.. 0r ..10..) in the pscd string. Example: 00010010.           *)
            for i := 2 to GetFdpth do
            BEGIN
   // Is there a sign change (0=>1 of 1=>0) at position i in the position-code?
              if (BitSet(i,GetFpscd) <> BitSet(i-1,GetFpscd)) then
              BEGIN // concat prnstr after s with the graphical element │ in between
                prnstr := Concat(s+vrt,prnstr);  { place the prnstr behind s }
                intern := TRUE;       { thus falls within the tree structure }

               {$IFDEF usedebug}
                LOGText(logfile,indent,'i='+IntToStr(i)+'): Bitswitch: so INTERN'+
                 '==> True and prnstr := (s+'+vrt+'+prnstr) => "'+prnstr+'"',0);
               {$ENDIF};
              END
           // no sign change
              else { if intern: concat prnstr after s with a space char  ...  }
                            {...instead of the graphical element │ in between }
                if intern then
                Begin
              //    prnstr := Concat(s+' ',prnstr);  //normal
                  prnstr := Concat(s+'+',prnstr); { + instead of space for logprint purposes }
                 {$IFDEF usedebug}
                  LOGText(logfile,indent,'i='+IntToStr(i)+'): INTERN=True: so '+
                                   'prnstr => (s,+,prnstr) => "'+prnstr+'"',0);
                 {$ENDIF};
                End;
            END;  { For i := 2 to GetFdpth }

           {$IFDEF usedebug}
            Dec(indent);
            LOGOUT(logfile,indent,' for i := 2 to '+IntToStr(GetFdpth)+' do');
           {$ENDIF};

            if intern then
            begin
              write(LST,prnstr);

             {$IFDEF usedebug}
              LOGText(logfile,indent,'Intern = TRUE ==> write prnstr: "'+prnstr+'"',0);
              LOGText(logfile,indent,'(so far printed line => "'+holds+prnstr+'")',0);
             {$ENDIF};
            end;  { if intern }

          END;  { if (trdpth-n) <> 0 then With reflink do }

      END; { if reflink.Givedpth >= 0 }

      writeln(LST);

     {$IFDEF usedebug}
      holds := '';
      LOGText(logfile,indent,'',0);
      LOGText(logfile,indent,'Writeln, so start of a new printline:',0);
      LOGText(logfile,indent,'INNIT: Holds := NulStr',1);
      LOGText(logfile,indent,'Now: goto Down node point:',1);
      Str(reflink.GetFnmcd:3,prnnmcd);
      LOGIn(logfile,indent,prnnmcd+' = Inorder(reflink.FDn,'+ IntToStr(trdpth-depth)+')');
     {$ENDIF};

      Inorder(reflink.GetFdn,depth-1);{ print lower part of the schedule tree }

     {$IFDEF usedebug}
      Str(reflink.GetFnmcd:3,prnnmcd);
      LOGOUT(logfile,indent,prnnmcd+' = Inorder(reflink.FDn,'+ IntToStr(trdpth-depth)+')');
     {$ENDIF};

    END  { If reflink <> Nil }
   {$IFDEF usedebug}
    else // only for logging
      if reflink = NIl then
        LOGText(logfile,indent,'..This reflink = NIl: limit of tree structure has been reached',0);
   {$ENDIF};

  END; { Inorder() }

BEGIN  { printtree }
(* Innit parameters *)
  trdpth := Givetrdpth;                                     { give tree depth }
  cntnd := 0;                                       { initiation node counter }
  s[0] := Chr(High(s) );          { Chr(High(s)) zet max lengte van s in s[0] }
  Fillchar(s[1],High(s),'.');             { fills entire s string with spaces }

 {$IFDEF usedebug}
  LOGINIT(datapad,'LOG-PrnLogTr.txt', logfile, 'Blaise Pascal Magazine Blaise_86_UK',true);
  holds := '';                                                        { innit }
  LOGText(logfile,indent,'start Printing Tree Schedule',1);           { start }
  LOGText(logfile,indent,'Innit S (fills entire showstring with spaces)==> "'+s+'"',0);{ show s }
  LOGText(logfile,indent,'Innit Holds ==> NulStr',1);           { show holds }
  LOGText(logfile,indent,'start Inorder(treestart, search depth: '+
                                                         IntToStr(trdpth)+')',1);
//  LOGText(logfile,indent,'',1);
 {$ENDIF};

 { print the entire tree structure recursively }
  Inorder(GetFRoot,trdpth);
  write(LST,'Number of nodes: ',cntnd);

 {$IFDEF usedebug}
  LogText(logfile,indent,'Number of nodes: '+IntToStr(cntnd),1);
  LOGText(logfile,indent,'End Printing Tree',1);
//  LOGText(logfile,indent,'',1);
  LOGDONE(logfile);
 {$ENDIF};
END;                                                           { TTr.prnlogtr}
(**------------------------------------------------------------------------**)

end.

