(* Program print schedule Tree for a Badminton knockout tournament
   made by: Rik Smit for Blaise Pascal Magazine 86_UK May 2020
   date dec 2019
 * Standard Lazarus 2.0.6 Tree Class definition                              *
 * Based on the 1993 DOS pascal program tree.pas by Rik Smit                 *
 * developed by Rik Smit  2019/2020                                          *
 * last update 27 mrt 2019, dec 2019                                         *
 *---------------------------------------------------------------------------*)

unit UTreeNode;

{$mode objfpc}{$H+}

interface

uses
  SysUtils, Forms, Controls, Dialogs, StdCtrls, ExtCtrls,
  utrclass;

type

  { TForm1 }

  TForm1 = class(TForm)
    BtnShowCanvas: TButton;
    BuildLogFiles: TButton;
    btnPrTr: TButton;
    Edit1: TEdit;
    lblcntnds: TLabel;
    lbltrdpth: TLabel;
    Panel1: TPanel;
    StaticText1: TStaticText;
    StaticText2: TStaticText;
    StaticText3: TStaticText;
    procedure btnPrTrClick(Sender: TObject);
    procedure BtnShowCanvasClick(Sender: TObject);
    procedure BuildLogFilesClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCreate(Sender: TObject);
  private

  public

  end;

var
  Form1: TForm1;
  Tree : TTr;
  ExePath : string;
{ Global var's for tracking memmory use /itemno; see heaph.inc file }
//  memrest,memrc : QWord;
  memstr        : string;
  memstartU,
  memstopU,
  memrest: QWord;

  myheapstatus  : TFPCHeapStatus;



implementation

uses  Std_log,
      DisplayForm;

{$R *.lfm}
{$DEFINE usedebug}  //self-defined compiler directive for the use of the logfile

Type str_20 = string[20];

(* ter info see also heaph.inc file

type
  TFPCHeapStatus = record
    MaxHeapSize,
    MaxHeapUsed,
    CurrHeapSize,
    CurrHeapUsed, ==> this is used by checking memory leaking
    CurrHeapFree  : ptruint;
  end;
  {$ifdef CPU64}
    PtrUInt = QWord;
  {$ifdef CPU32}
  PtrUInt = DWord;
*)
{ TForm1 }
var
  logfile : Text;

procedure TForm1.BtnShowCanvasClick(Sender: TObject);
var
  i :integer;
{ Local var's for tracking memmory use /itemno; see heaph.inc file }
  memstart,
  memstop: QWord;
  logfile : Text;
begin
 {$IFDEF usedebug}
  myheapstatus := GetFPCHeapStatus;
  memstart :=  myheapstatus.CurrHeapUsed;
  memstr := IntToStr(memstart);
  LOGINIT(datapad,'LOG-Canvas.txt', logfile, datapad,true);
  LOGIn(logfile,indent,'memory by start BtnShowCanvasClick: '+memstr);
 {$ENDIF};

  i := StrToInt(edit1.text);
  If i > 8 then
  Begin
    ShowMessage('Max Treedepth is 8!');
    i := 8;
    edit1.Text := IntToStr(i);
  end;

  If tree <> nil then tree.destroy;  // clean up any old tree first
  tree := TTr.Create;
  tree.SetFroot(tree.MakeTree(i));

  lbltrdpth.Caption := inttostr(tree.Givetrdpth);
  lblcntnds.Caption := IntToStr(tree.CntNds);

  If frmdspl = Nil Then  frmdspl := Tfrmdspl.Create(Self);
  frmdspl.show;

  frmdspl.Caption:= Concat('Depth tree = ',lbltrdpth.Caption,'  Nodes count = ',lblcntnds.Caption);

  frmdspl.Refresh;

 {$IFDEF usedebug}
  myheapstatus := GetFPCHeapStatus;
  memstop :=  myheapstatus.CurrHeapUsed;
  memrest := (memstart-memstop);
  memstr := IntToStr(memrest);
  LOGOUT(logfile,indent,'Memory at the end of BtnShowCanvasClick: '+memstr);
  LOGDONE(logfile);
 {$ENDIF};
end;                              {TForm1.BtnShowCanvasClick(Sender: TObject);}

procedure TForm1.btnPrTrClick(Sender: TObject);
var
  i :integer;
begin
  i := StrToInt(edit1.text);
  If i > 8 then
  Begin
    ShowMessage('Max Treedepth is 8!');
    i := 8;
    edit1.Text := IntToStr(i);
  end;

  If tree <> nil then tree.destroy;  // clean up any old tree first
  tree := TTr.Create;
  tree.SetFroot(tree.MakeTree(i));

  lbltrdpth.Caption := inttostr(tree.Givetrdpth);
  lblcntnds.Caption := IntToStr(tree.CntNds);

  assignFile(LST,(datapad+'Print-tree.txt'));
  try
    rewrite(LST);
    tree.printtree;
  finally
    CloseFile(LST);
  end;
end;

procedure TForm1.BuildLogFilesClick(Sender: TObject);
var i :integer;
begin
  i := StrToInt(edit1.text);
  If i > 8 then
  Begin
    ShowMessage('Max Treedepth is 8!');
    i := 8;
    edit1.Text := IntToStr(i);
  end;

  If tree <> nil then
    tree.destroy;     // clean up any old tree first
  tree := TTr.Create;
  tree.SetFRoot(tree.MakeTree(i));
  lbltrdpth.Caption := inttostr(tree.Givetrdpth);
  lblcntnds.Caption := IntToStr(tree.CntNds);
  assignFile(LST,(datapad+'Print-tree-LOG.txt'));
  try
    rewrite(LST);
  // tree.printtree;
    tree.PrnLogTr;
  finally
    CloseFile(LST);
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  exepath := GiveEXEPath;  // Path Dos pascal
  exepath := ExtractFilePath(Application.ExeName) ; // Path Lazarus
  datapad := exepath+ '\output\';
//  datapad := 'g:\Lazarus\testprojecten\classtrnd\output\';
 {$IFDEF usedebug}
  LOGINIT(datapad,'LOG-UTreeNode.txt', logfile,'LOG UTreeNode',true);
  myheapstatus := GetFPCHeapStatus;
  memstartU :=  myheapstatus.CurrHeapFree;
  memstr := IntToStr(memstartU);
  LOGIn(logfile,indent,'Heap memory at start UTreeNode: '+memstr);
 {$ENDIF};
end;

procedure TForm1.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin
  if tree <> NIl then tree.logdestroy;
 {$IFDEF usedebug}
  myheapstatus := GetFPCHeapStatus;
  memstopU :=  myheapstatus.CurrHeapFree;
  memrest := (memstopU-memstartU);
  memstr := IntToStr(memrest);
  LOGOUT(logfile,indent,'Lost Heap memory at end UTreeNode: '+memstr);
  LOGDONE(Logfile);
 {$ENDIF};
end;




end.

