{*-- UTrNdClass.pas --------------------------------------------30-01-2019----*
 * By E.E. Smit
 * Standaard Tree-knoop Class definitie                                       *
 * ontwikkeld op 20 februari 1993 voor Turbo Pascal                           *
 * Omgezet naar Lazarus FreePascal 1 maart 2019                               *
 * laatste update 27 mrt 2019, 4/23 apr 2019                                  *
 *----------------------------------------------------------------------------*}
unit UTrNdClass;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils,lazUTF8;

type

{ TTrNd = Type Tree Node}

TTrNd = class(TObject)
private
  Fdpth: Byte;                                     { tree diepte tree branches}
  Fpscd: Byte;                          { tree positie code, bit georienteerd }
  Fnmcd: Byte;                                             { tree nummer code }
  Fup : TTrNd;                                          {forwards tree-up link}
  Fdn : TTrNd;                                          {forwards tree-dn link}
  Frt : TTrNd;                                       {Backwards tree-root link}

public
  constructor Create(Adpth, Apscd, Anmcd : Byte);
  destructor  Destroy;  virtual;

  function    GetFdpth : Byte;
  function    GetFpscd : Byte;
  function    GetFnmcd : Byte;
  function    GetFup   : TTrNd;
  function    GetFdn   : TTrNd;
  function    GetFrt   : TTrNd;

  procedure   SetFdpth(Adpth: Byte);
  procedure   SetFpscd(Apscd: Byte);
  procedure   SetFnmcd(Anmcd: Byte);
  procedure   SetFup  (ANode: TTrNd);
  procedure   SetFdn  (ANode: TTrNd);
  procedure   SetFrt  (ANode: TTrNd);

  function    NdInfoStr:String;
//  function    NdInfoStr(mxdpth: word; var s: OpenString):String;
end;  { TTrNd = Class()}

(**  Info OpenString  april 2019 oa DR Bob
syntax: Subroutine declaration(....; var Param: OpenString; ..);

$P or $OPENSTRINGS : Use open strings
They can be declared in two ways, either by using the OpenString identifier or
with the String type in combination with the {$P+} compiler option. For an
open-string parameter, the actual parameter given to a procedure or function
can be of any string size. Within the procedure or function, the maximum length
of the formal parameter will be the same as that of the actual parameter.
Delphi will take care of possible 'overflow' situation by truncating the strings
where necessary (without the danger of a stack overflow, with the {$V} option in
the Borland Pascal days).

If this switch is on, all function or procedure parameters of type string are
considered to be open string parameters; this parameter only has effect for
short strings, not for ansistrings.

When using openstrings, the declared type of the string can be different from
the type of string that is actually passed, even for strings that are passed by
reference. The declared size of the string passed can be examined with the
High(P) call.

By default, the use of openstrings is off.

voorbeeld
    procedure CopyStr(var OutStr,InStr: OpenString);
    begin
      OutStr := InStr
    end {CopyStr};

  In the example above, the InStr will be truncated to the maximum allowed length
  of OutStr, which can be seen in the next example:

    program Hood3;
    uses WinCrt;

      procedure CopyStr(var OutStr,InStr: OpenString);
      begin
        OutStr := InStr
      end {CopyStr};

    var S1: String[30];
        S2: String[21];
    begin
      S1 := 'This is an OpenString Example!';
      CopyStr(S2,S1);
      writeln(S1) { S1 = 'This is an OpenString' }
    end.

Open-string parameters behave like normal variable parameters of type string,
except that they cannot be passed as regular variable parameters to other
procedures or functions. They can only be passed as open-string parameters again.
**)

binstr = string[8];//wordt de byte pscd in 8 bits mee weergegeven bv: '01100101'

(** Onderstaande functies worden gebruikt om individuele bits in
    een byte te manupuleren                                      **)

function BitSet      (pos, item: byte)       :Boolean;
function SetBit      (pos, item: byte)       :Byte;
function ClearBit    (pos, item: byte)       :Byte;
function GiveRank    (Apscd: Byte; Adpth: Byte) :Word;
function GiveRankPscd(Arnk : byte; Adpth: Byte) :Word;
(* givebinstr: b is het om te zetten byte, d geeft het aantal bits
               dat in de string gezet moet worden. Max 8          *)
function GiveBinstr  (b, d: byte)             :binstr;

implementation
//{$DEFINE usedebug}    // Zelf gedefinieerde compiler directive tbv logfile

(**  TTrNd  Class definitions --------------------------------------------**)

constructor TTrNd.Create(Adpth, Apscd, Anmcd : Byte);
begin
  Fdpth  := Adpth;
  Fpscd  := Apscd;
  Fnmcd  := Anmcd;
  Fup    := Nil;
  Fdn    := Nil;
  Frt    := Nil;
end;
(**------------------------------------------------------------------------**)
destructor TTrNd.Destroy;
begin
  Inherited;
end;

(**-- GET ------------------------------------------------------------------**)
FUNCTION TTrnd.GetFDpth: Byte;
BEGIN
  result := Fdpth;
END;
(**------------------------------------------------------------------------**)
FUNCTION TTrnd.GetFPscd: Byte;
BEGIN
  result := Fpscd;
END;
(**------------------------------------------------------------------------**)
FUNCTION TTrnd.GetFNmcd: Byte;
BEGIN
  result := Fnmcd;
END;
(**------------------------------------------------------------------------**)
FUNCTION TTrnd.GetFup: TTrNd;
BEGIN
  result := Fup;
END;
(**------------------------------------------------------------------------**)
FUNCTION TTrnd.GetFdn: TTrNd;
BEGIN
  result := Fdn;
END;
(**------------------------------------------------------------------------**)
FUNCTION TTrnd.GetFrt: TTrNd;
BEGIN
  result := Frt;
END;

(**-- Set -----------------------------------------------------------------**)
PROCEDURE TTrnd.SetFdpth(Adpth: Byte);
BEGIN
  Fdpth := Adpth;
END;
(**------------------------------------------------------------------------**)
PROCEDURE TTrnd.SetFpscd(Apscd: Byte);
BEGIN
  Fpscd := Apscd;
END;
(**------------------------------------------------------------------------**)
PROCEDURE TTrnd.SetFNmcd(Anmcd: Byte);
BEGIN
  Fnmcd := Anmcd;
END;
(**------------------------------------------------------------------------**)

PROCEDURE TTrnd.SetFup (ANode: TTrNd);
BEGIN
  Fup := ANode;
END;
(**------------------------------------------------------------------------**)
PROCEDURE TTrnd.SetFdn (ANode: TTrNd);
BEGIN
  Fdn := ANode;
END;
(**------------------------------------------------------------------------**)
PROCEDURE TTrnd.SetFrt (ANode: TTrNd);
BEGIN
  FRt := ANode;
END;
(**------------------------------------------------------------------------**)

(** NdInfoStr geeft de beschikbare informatie van het knooppunt als volgt weer:
(,aanmaak volgorde nummer, ), oplopend volgorde nummer vh schema op dat niveau,
[, treediepte, -, nmcd uitgeschreven als binstring, =, nmcd als getal waarde, ]
 vb maxdpth 7: dpth 7, lotings knoop 24 geeft: (53) 24[7-1110100=116]
 vb maxdpth 7: dpth 4,    tussenknoop 7 geeft: (100) 7[4-0110=6]
 vb maxdpth 4: dpth 4, lotings knoop  7 geeft: (16)  7[4-0110=6]
 vb maxdpth 4: dpth 2,    tussenknoop 3 geeft: (18)  3[2-01=1]
 vb maxdpth 4: dpth 0,    tussenknoop 1 geeft: (nr1) 1[0] Finale            **)

FUNCTION TTrnd.NdInfoStr:String;
  var  prndpth : string[2];
       prnrank : string[3];
       prnpscd : string[3];
       prnnmcd : string[3];
BEGIN
(* print strings klaar zetten *)
  Str(GiveRank(Fpscd,Fdpth):3,prnrank); // :3 is ivm uitlijnen
  Str(GetFdpth,prndpth);
  Str(GetFpscd,prnpscd);
  Str(GetFnmcd,prnnmcd);

  if GetFdpth = 0 then                                            // finale
    Result := Concat('(nr',prnnmcd,')',prnrank,'[',prndpth,']',' Finale.')
  else
    Result := Concat('(',prnnmcd,')',prnrank,'[',prndpth,
              '-',GiveBinstr(GetFpscd,GetFdpth),'=',prnpscd,']');
END;                                                       // TTrnd.NdInfoStr
(**-------------------------------------------------------------------------**)

(** 3 Functies om individuele bits in een byte mee te manupuleren date:1993-**)

  (**----------------------------------------------------------------*
    * BitSet ? kijkt of de opgegeven positie ( POS ) in de opgegeven *
    *          variable ( ITEM ) de waarde 1 heeft.                  **)
FUNCTION BitSet(pos: byte; item: byte): Boolean;
  var bt : Byte;
BEGIN
  bt    := $01;                                       { bt krijgt 00000001 }
  if Pos > 1 then                                             { begrenzing }
    bt    := bt SHL (Pos-1);         { verplaats bit no 1 pos-1 naar links }
  Result := (item and bt) > 0;                 { is bij beiden bit gezet ? }
END;
(**------------------------------------------------------------------------**)

  (** SetBit zet het bit op de opgegeven positie ( POS ) *
    *        in de opgegeven variable ( ITEM ) op 1.     **)
FUNCTION SetBit(pos, item: byte): Byte;
  var bt : Byte;
 BEGIN
  bt := $01;
  bt := bt SHL (Pos-1);
  Result := item or bt;
END;
(**------------------------------------------------------------------------**)

  (** ClearBit zet het bit op de opgegeven positie ( POS ) *
    *          in de opgegeven variable ( ITEM ) op 0.     **)
FUNCTION ClearBit(pos, item: byte): Byte;
  var bt : Byte;
BEGIN
  bt := $01;
  if Pos > 1 then                                               { begrenzing }
    bt := bt SHL (Pos-1);              { verplaats bit no 1 pos-1 naar links }
  if (item and bt) > 0 then
(* bit is gezet en kan dus op nul worden gezet *)
    Result := (item xor bt)
  else Result := item;
END;
(**------------------------------------------------------------------------**)

(** GiveRank zet de pscd byte waarde om in een treediepte afhankelijke rank- *
*            ing waarde die van boven naar beneden oplopend genummerd is,    *
*            te beginnen met de waarde 1.                   date: 1993     **)
FUNCTION GiveRank(Apscd, Adpth: Byte): Word;
  var i,j : Word; { tellers }
BEGIN
  j := 0;
 (** Indien van Hi to Lo het bit in pscd-byte is gezet deze als      *
 *   byte-waarde gelezen waarde sommeren bij het functie resultaat **)
  for i:= Adpth downto 1 do      // bitwaarde van rechts naar links lezen!
    if BitSet(i,Apscd) then Inc(j,SetBit((Adpth-i+1),0));
  Result := j+1;                   {bij resultaat startnulwaarde compenseren }
END;                                                              { GiveRank }
(**------------------------------------------------------------------------**)

(** GiveRankPscd zet de ranking waarde om in een treediepte afhankelijke     *
*                pscd byte waarde.                          dd 13 okt 2003 **)
FUNCTION GiveRankPscd(Arnk, Adpth: Byte): Word;
  var i : Word; // teller
BEGIN
  Result := 0;
  DEC(Arnk);                      { start nulwaarde compenseren zie GiveRank }
  for i := 1 to Adpth do
  BEGIN
    if BitSet(i,Arnk) then       // bitwaarde van links naar rechts lezen
    Result := SetBit(( Adpth-i+1),Result);
  END;
END;                                                              { GiveRank }
(**------------------------------------------------------------------------**)

  (** GiveBinstr zet een byte om in een string eenen en nullen.
      givebinstr: b is het om te zetten byte, d geeft het aantal bits
                  dat in de string gezet moet worden. Max 8            **)
FUNCTION GiveBinstr(b,d: byte): binstr;
  var i, bt : byte;
      s : binstr;
BEGIN
  s := '';
  bt := $01;
  for i := 1 to d do
  BEGIN
    if (b and bt) > 0 then s := '1'+s   // er staat daar een 1 in b
                      else s := '0'+s;  // er staat daar een 0 in b
    bt := bt shl 1;  // bit plaats 1 positie naar links opschuiven
  END;
  Result := s;
END;                                                            { GiveBinstr }
(**------------------------------------------------------------------------**)

end.



